---------------------------------------------------------------------------
--                                                                       --
--  Ast2Cfg is an ASIS application that generates a CFG out of the AST.  --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--     Copyright (c) 2007, 2008, 2009 Raul Fechete, Georg Kienesberger   --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of Ast2Cfg.                                         --
--                                                                       --
-- Ast2Cfg is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- Ast2Cfg is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Exceptions;
with Ada.Containers;

package Ast2Cfg.Id is

   type Pointer is private; -- a pointer to an id object, only way to reference it

   function New_Pkg_Id return Pointer; -- get an id for a new package
   function New_Cfg_Id return Pointer; -- get an id for a new cfg
   function New_Node_Id return Pointer; -- get an id for a new node

   -- it is not allowed/possible to declare a plain id object variable anywhere outside
   -- this package, therefore you need to call one of the above functions to get a new
   -- id.

   function  Get_Pkg_Num  (Id: in Pointer) return Long_Long_Natural; -- get the pkg part of an id
   function  Get_Cfg_Num  (Id: in Pointer) return Long_Long_Natural; -- get the cfg part of an id
   function  Get_Node_Num (Id: in Pointer) return Long_Long_Natural; -- get the node part of an id

   function "=" (Left: in Pointer; Right: in Pointer) return Boolean; -- test for equality of 2 ids,
                                                                      -- /= is implicitly available

   function "<" (Left: in Pointer; Right: in Pointer) return Boolean;

   function ">" (Left: in Pointer; Right: in Pointer) return Boolean;

   function To_String (Id: in Pointer) return String; -- get a string representation of the id
   function To_CFG_Name (Id: in Pointer) return String; -- generate a CFG Name out of the id
   function Hash (Id: in Pointer) return Ada.Containers.Hash_Type; -- return the hash of an id

   procedure Free (Id: in out Pointer); -- destroy the id, set the pointer to null

   Pkg_Overflow, Cfg_Overflow, Node_Overflow: exception; -- too many packages, CFGs, nodes

private

   Pkgs:  Long_Long_Natural := 0;
   Cfgs:  Long_Long_Natural := 0;
   Nodes: Long_Long_Natural := 0;

   type Object is limited
      record
         Pkg:  Long_Long_Natural := Pkgs;
         Cfg:  Long_Long_Natural := Cfgs;
         Node: Long_Long_Natural := Nodes;
      end record;

   type Pointer is access Object;

end Ast2Cfg.Id;
