---------------------------------------------------------------------------
--                                                                       --
--  Ast2Cfg is an ASIS application that generates a CFG out of the AST.  --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--     Copyright (c) 2007, 2008, 2009 Raul Fechete, Georg Kienesberger   --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of Ast2Cfg.                                         --
--                                                                       --
-- Ast2Cfg is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- Ast2Cfg is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Unchecked_Deallocation;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

with Asis;

with Ast2Cfg.Generic_List;
with Ast2Cfg.Flow_Types; use Ast2Cfg.Flow_Types;
with Ast2Cfg.CFGs; use Ast2Cfg.CFGs;
with Ast2Cfg.Id;

package Ast2Cfg.Pkgs is

   type Pkg_Object     is abstract new Flow_Object with private;
   type Pkg_Class_Ptr  is access all Pkg_Object'Class;

   package Pkg_Class_Ptr_List is new Generic_List (Item => Pkg_Class_Ptr);

   type Body_Object      is abstract new Pkg_Object with private;
   type Spec_Object      is abstract new Pkg_Object with private;

   type Def_Object         is new Body_Object with private;
   type Simple_Body_Object is new Body_Object with private;
   type Prot_Object        is new Body_Object with private;

   type Simple_Spec_Object is new Spec_Object with private;
   type Gen_Object         is new Spec_Object with private;
   type Single_Prot_Object is new Spec_Object with private;
   type Prot_Type_Object   is new Spec_Object with private;

   not overriding -- for overriding Init see private part
   procedure Init         (Pkg:  in out Pkg_Object;
                           Name: in Unbounded_String;
                           Pred: in Flow_Class_Ptr;
                           Element: in Asis.Element);

   not overriding
   procedure Free (Pkg: in out Pkg_Class_Ptr);

   not overriding
   procedure Free_Special (Pkg: in out Pkg_Class_Ptr);

private

   overriding
   procedure Init (Pkg:  in out Pkg_Object;
                   Oid:  in Id.Pointer;
                   Name: in Unbounded_String;
                   Pred: in Flow_Class_Ptr;
                   Element: in Asis.Element);

   type Pkg_Object  is new Flow_Object with
     null record;

   type Body_Object is new Pkg_Object with
     null record;

   type Spec_Object is new Pkg_Object with
     null record;

   type Def_Object is new Body_Object with
     null record;

   type Simple_Body_Object is new Body_Object with
     null record;

   type Prot_Object is new Body_Object with
     null record;

   type Simple_Spec_Object is new Spec_Object with
     null record;

   type Gen_Object is new Spec_Object with
     null record;

   type Single_Prot_Object is new Spec_Object with
     null record;

   type Prot_Type_Object is new Spec_Object with
     null record;

end Ast2Cfg.Pkgs;
